<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly

class WC_PayPlus_Admin_Payments {

	protected static $instance = null;

	public static function get_instance() {
		if ( ! isset( static::$instance ) ) {
			static::$instance = new static;
		}

		return static::$instance;
	}

	public $error_msg = 0;

	private function __construct() {
		// admin order page css
		global $pagenow;

		if ( 'post.php' === $pagenow && isset( $_GET['post'] ) && 'shop_order' === get_post_type( $_GET['post'] ) ) {
			add_action( 'admin_enqueue_scripts', [ $this, 'load_admin_assets' ] );
		}

		// make payment button for j2\j5
		add_action( 'woocommerce_order_actions_end', [ $this, 'make_payment_button' ], 10, 1 );

		// process make payment
		add_action( 'save_post_shop_order', [ $this, 'process_make_payment' ], 10, 1 );

		// admin notices
		add_action( 'admin_notices', [ $this, 'admin_notices' ], 15 );

		// remove query args after error shown
		add_filter( 'removable_query_args', [ $this, 'add_removable_arg' ] );
		/* 
		// update payments on admin edit order
		add_action( 'woocommerce_saved_order_items', [ $this, 'after_order_update' ], 10, 2 ); */
	}

	public function load_admin_assets() {
		wp_enqueue_style( 'payplus', PAYPLUS_PLUGIN_URL . 'assets/css/admin.min.css', [], PAYPLUS_VERSION );
		wp_enqueue_script( 'payplus-admin-payment', PAYPLUS_PLUGIN_URL . '/assets/js/admin-payments.js', [ 'jquery' ], PAYPLUS_VERSION, true );
	}

	public function make_payment_button( $post_id ) {
		if (get_post_meta( $post_id, 'payplus_type', true) == "Charge") return;
		$order = wc_get_order( $post_id );
		$total = $order->get_total();

		$token = get_post_meta( $post_id, 'payplus_token_uid', true );
		if ( ! $token || $token == '' ) {
			return;
		}
		echo "<li class='wide delayed-payment'>
			<h3>" . __( 'Charge Order Using PayPlus', 'payplus-payment-gateway' ) . "</h3>
			<span id='waitingChargeMessage'>
				<input type='number' id='totalorder' name='payplus_charge_amount' value='".$total."' min='0' max='".$total."' step='0.01' required />
				<button type='submit' name='payplus-token-payment' class='button button-primary'><span class='dashicons dashicons-cart'></span> ".__( 'Make Payment', 'payplus-payment-gateway' )."</button>
			</span>
		</li>";
		echo '<script>
		jQuery("button[name=payplus-token-payment]").on(\'click\', function() {
			jQuery("button[name=payplus-token-payment]").html("<span class=\'dashicons dashicons-cart\'></span> '.__( 'Please wait', 'payplus-payment-gateway').'...");
			setTimeout(function(){
				jQuery("button[name=payplus-token-payment]").prop("disabled",true);
				},1
				);
			});
		</script>';
	}

	public function process_make_payment( $order_id ) {
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) return;
		if ( ! isset( $_POST['payplus-token-payment'] ) ) return;
		$WC_PayPlus_Gateway = new WC_PayPlus_Gateway();
		$order = wc_get_order( $order_id );
		if ($WC_PayPlus_Gateway->logging) $logger = wc_get_logger();
		$charged_amount=0;
		$charged_amount = (double)$order->get_meta('payplus_charged_j5_amount');
		if ($charged_amount) return;
		$OrderType = $order->get_meta('payplus_type');
		$chargeByItems = false;
		$amount = round((double)$_POST['payplus_charge_amount'],2);
		$transaction_uid = $order->get_meta('payplus_transaction_uid');
		if ($OrderType == "Charge") return;
		
		if ($amount > $order->get_total()) {
			if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'Cannot Charge then original order');
			$order->add_order_note( sprintf( __( 'Cannot Charge then original order sum', 'payplus-payment-gateway' ), $charged_amount, $order->get_currency() ) );
			return false;
			}
		
		if (!$amount OR $amount <= 0 OR $charged_amount > 0) {
			$order->add_order_note( sprintf( __( 'The charge in PayPlus already made. Please check your PayPlus account<br />Amount: %s %s', 'payplus-payment-gateway' ), $charged_amount, $order->get_currency() ) );
			return false;
			}
		
		if ($OrderType != "Approval") {
			if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'Transaction Not J5 Or Changed to J4 After Charge');
			return false;
			}
		
		if ($amount == $order->get_total() AND $charged_amount == 0) {
			$chargeByItems = true;
			if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'Sending Items...');
		
			$totalCartAmount = 0;
			$disCountPrice = 0;
			
			foreach ( $order->get_items(['line_item', 'fee', 'coupon']) as $item => $item_data ) {
				$quantity = ($item_data['quantity'] ? $item_data['quantity'] : '1');
				if ($item_data['type'] == "coupon") {
					$productPrice = '-'.round( ($item_data['line_total']+$item_data['line_total_tax']), $WC_PayPlus_Gateway->rounding_decimals );
					$totalCartAmount -= ($productPrice*$quantity);
					} else if ($item_data['type'] == "fee") {
					$productPrice = round( ($item_data['line_total']+$item_data['line_total_tax']), $WC_PayPlus_Gateway->rounding_decimals );
					$totalCartAmount += ($productPrice*$quantity);
					} else {
					$productPrice = round( ($item_data['line_subtotal']+$item_data['line_subtotal_tax']) / $item_data['quantity'], $WC_PayPlus_Gateway->rounding_decimals );
					$totalCartAmount += ($productPrice*$quantity);
					}
				
				$product = new WC_Product($item_data['product_id']);
				$productSKU = $product->get_sku();
				
				$productsItems[] = '{	
					"name" : "'.str_replace( [ "'", '"' ], '', $item_data['name']).'",
					"barcode": "'.($productSKU ? $productSKU : $item_data['product_id']).'",
					"quantity" : '.$quantity.',
					"price" : '.$productPrice.'
					}';
			}
			
			// order shipping
			if ( $order->get_shipping_total() ) {
				foreach ( $order->get_shipping_methods() as $shipping_method ) {
					$shipping_method_data = $shipping_method->get_data();
					$totalLine = round( $shipping_method_data['total'],$WC_PayPlus_Gateway->rounding_decimals);
					$productsItems[] = '{	
						"name" : "'.__('Shipping', 'payplus-payment-gateway').' - ' . str_replace( [ "'", '"' ], '', $shipping_method_data['name'] ).'",
						"quantity" : 1,
						"price" : '.$totalLine.'
						}';
					$totalCartAmount += $totalLine;
				}
			}
			
			if ( $total_discount = $order->get_discount_total() ) {
				$disCountPrice = round( $total_discount + $order->get_discount_tax(), $WC_PayPlus_Gateway->rounding_decimals);
				$productsItems[] = '{	
						"name" : "'.__('Discount', 'payplus-payment-gateway').'",
						"quantity" : 1,
						"price" : -'.$disCountPrice.'
						}';
				$totalCartAmount -= $disCountPrice;
				}
			}
		
		$payload = '{
				"transaction_uid": "'.$transaction_uid.'",
				"amount": "'.($chargeByItems ? $totalCartAmount : $amount).'",
				'.($chargeByItems ? '
				"items": [
					'.implode(",",$productsItems).'
					]' : '
				"more_info": "'.__('Charge for Order Number: ','payplus-payment-gateway').$order_id.'"
				').'
				}';
		
		if ($WC_PayPlus_Gateway->api_test_mode) $apiURL = 'https://restapidev.payplus.co.il/api/v1.0/';
			else $apiURL = 'https://restapi.payplus.co.il/api/v1.0/';

		if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'Sending Request: ' .  print_r( $payload, true ));
		
		$response = $this->post_payplus_ws($apiURL.'Transactions/ChargeByTransactionUID',$payload);
		$res=json_decode(wp_remote_retrieve_body($response));
		if ($res->results->status == "success" && $res->data->transaction->status_code == "000") {
			update_post_meta( $order_id, 'payplus_charged_j5_amount', $amount );
			update_post_meta( $order_id, 'payplus_type', 'Charge' );
			update_post_meta( $order_id, 'payplus_number', $res->data->transaction->number );
			update_post_meta( $order_id, 'payplus_transaction_uid', $res->data->transaction->uid );
			update_post_meta( $order_id, 'payplus_status_code', $res->data->transaction->status_code );
			update_post_meta( $order_id, 'payplus_number_of_payments', $res->data->transaction->payments->number_of_payments );
			update_post_meta( $order_id, 'payplus_first_payment_amount', $res->data->transaction->payments->first_payment_amount );
			update_post_meta( $order_id, 'payplus_payments_rest_payments_amount', $res->data->transaction->payments->rest_payments_amount );
			update_post_meta( $order_id, 'payplus_auth_num', trim( $res->data->transaction->approval_number ) );
			update_post_meta( $order_id, 'payplus_voucher_num', trim( $res->data->transaction->voucher_number ) );
			
			$order->add_order_note( sprintf( __( 'PayPlus Charge is Successful<br />Charge Transaction Number: %s<br />Amount: %s %s', 'payplus-payment-gateway' ), $res->data->transaction->number, $res->data->transaction->amount, $order->get_currency() ) );
			if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'WP Charge Remote Post Completed: ' .  print_r( $res, true ));
			$_POST['order_status'] = $order->needs_processing() ? 'wc-processing' : 'wc-completed';
			$order->payment_complete();
			} else {
			$order->add_order_note( sprintf( __( 'PayPlus Charge is Failed<br />Status: %s<br />Description: %s', 'payplus-payment-gateway' ), $res->results->status, $res->results->description ) );
			if ($WC_PayPlus_Gateway->logging) $logger->add( 'payplus_process_j5_payment', 'WP Charge Remote Post Error: ' . print_r( $res, true ) );
			$this->error_msg = 2;
			}
		add_filter( 'redirect_post_location', [ $this, 'add_notice_query_var' ], 99 );
	}

	public function post_payplus_ws($url,$payload) {
		$WC_PayPlus_Gateway = new WC_PayPlus_Gateway();
		$args = array(
			'body' => $payload,
			'timeout'     => '5',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking'    => true,
			'headers'     => [],
			'headers' => array(
				'Content-Type' => 'application/json',
				'Authorization' => '{"api_key":"'.$WC_PayPlus_Gateway->api_key.'","secret_key":"'.$WC_PayPlus_Gateway->secret_key.'"}'
				)
			);
		$response = wp_remote_post( $url, $args );
		return $response;
		}
	
	public function add_notice_query_var( $location ) {
		remove_filter( 'redirect_post_location', [ $this, 'add_notice_query_var' ], 99 );

		return add_query_arg( [ 'error_msg' => $this->error_msg ], $location );
	}

	public function admin_notices() {
		if ( ! isset( $_GET['error_msg'] ) ) {
			return;
		}

		$title = __( 'PayPlus Payment Gateway', 'payplus-payment-gateway' );
		$class = 'notice-error';
		switch ( $_GET['error_msg'] ) {
			case 1:
				$message = __( 'user or other, please contact payplus support', 'payplus-payment-gateway' );
				break;
			case 2:
				$message = __( 'Credit card company declined, check credit card details and credit line', 'payplus-payment-gateway' );
				break;
			default:
				$message = __( 'PayPlus Payment Successful', 'payplus-payment-gateway' );
				$class   = 'notice-success';
		}
		$output = "<div class='notice $class is-dismissible'><p><b>$title:</b> $message</p></div>";

		echo $output;
	}
	
	// update payments on admin edit order
	public function after_order_update( $order_id ) {
		return false;
		/* $payments = get_post_meta( $order_id, 'payplus_number_of_payments', true );
		if ( $payments > 0 ) {
			$order = wc_get_order( $order_id );

			$total   = $order->get_meta( 'payplus_process_j5_payment' ) ? : $order->get_total();
			$amounts = $this->update_payments_amounts( $total, $payments );

			if ($amounts) {
				$order->update_meta_data( 'payplus_first_payment_amount', $amounts['first'] );
				$order->update_meta_data( 'payplus_payments_rest_payments_amount', $amounts['non_first'] );
				$order->save();
			}
		} */
	}
	/* 
	private function update_payments_amounts( $total, $payments ) {
		if ( ! $payments ) {
			return;
		}

		$payment_amount = $first_amount = number_format( $total / ( $payments + 1 ), 2 );
		$check_amount   = number_format( $payment_amount * $payments + $payment_amount, 2 );

		if ( $total - $check_amount !== 0 ) {
			$first_amount = number_format( $payment_amount + ( $total - $check_amount ), 2 );
		}

		return [
			'first'     => $first_amount,
			'non_first' => $payment_amount
		];
	} */
	
	public function add_removable_arg( $args ) {
		$args[] = 'error_msg';

		return $args;
	}

}

WC_PayPlus_Admin_Payments::get_instance();