<?php
defined('ABSPATH') || exit; // Exit if accessed directly

abstract class WC_PayPlus_Subgateway extends WC_PayPlus_Gateway {
    public $id;
    public $payplus_default_charge_method;
    public $iconURL;
    public $method_title_text;
    public $method_descrition_text;
    public $pay_with_text;
    public $default_description_settings_text;
    public function __construct()
    {
        parent::__construct();
        if ($this->hide_icon == "no") {
			$this->icon = PAYPLUS_PLUGIN_URL . $this->iconURL;
		}
        $this->method_title       = __($this->method_title_text, 'payplus-payment-gateway');
        $this->default_charge_method 	= $this->payplus_default_charge_method;
        add_action('woocommerce_receipt_' . $this->id, [$this, 'receipt_page']);
        if ($this->settings['enabled'] === null) {
            $this->enabled = 'no';
        }
    }

	public function init_form_fields()
	{
		$this->form_fields = [
			'title'             => [
				'title'       => __('Title', 'payplus-payment-gateway'),
				'type'        => 'text',
				'description' => __('This controls the title which the user sees during checkout', 'payplus-payment-gateway'),
				'default'     => __($this->pay_with_text, 'payplus-payment-gateway'),
				'desc_tip'    => true,
			],
			'description'       => [
				'title'   => __('Description', 'payplus-payment-gateway'),
				'type'    => 'textarea',
				'default' => __($this->default_description_settings_text, 'payplus-payment-gateway')
            ],			
            'display_mode'      => [
				'title'   => __('Display Mode', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'default' => __('Use global default', 'payplus-payment-gateway'),
					'redirect' => __('Redirect', 'payplus-payment-gateway'),
					'iframe'   => __('iFrame', 'payplus-payment-gateway'),
					'samePageIframe'   => __('iFrame on the same page', 'payplus-payment-gateway'),
					'popupIframe'   => __('iFrame in a Popup', 'payplus-payment-gateway'),
				],
				'default' => 'redirect',
			],
			'iframe_height'     => [
				'title'   => __('iFrame Height', 'payplus-payment-gateway'),
				'type'    => 'number',
				'default' => 700,
			],
            'hide_icon'           => [
				'title'   => __('Hide PayPlus Icon In The Checkout Page', 'payplus-payment-gateway'),
				'type'    => 'checkbox',
				'label'   => __('Hide PayPlus Icon In The Checkout Page', 'payplus-payment-gateway'),
				'default' => 'no'
			],
		];
        
	}
    
    public function admin_options() {
        parent::admin_options();
        echo __("Before enabling this option, please ensure you have proper PayPlus credentials and authorization",'payplus-payment-gateway');
    }

    public function init_settings()
    {   
        $defaultOptions = [
            'enabled' => 'no',
            'title'=>'',
            'description'=>'',
            'display_mode'=>'default',
            'iframe_height'=>700,
            'hide_icon'=>'no',
        ]; 
        $subOptionsettings = get_option($this->get_option_key(),$defaultOptions );

        $this->settings = get_option('woocommerce_payplus-payment-gateway_settings', $defaultOptions );
        $enabled = 'no';
        if ($subOptionsettings['enabled'] == 'yes' && $this->settings['enabled'] == 'yes') {
            $enabled = 'yes';
        }
        $this->settings['enabled'] = $enabled;
        
        $this->enabled = $enabled;
        $this->settings['description'] = $subOptionsettings['description'];
        $this->settings['title'] = $subOptionsettings['title'];
        $this->settings['display_mode'] = $subOptionsettings['display_mode'];
        $this->settings['hide_icon'] = $subOptionsettings['hide_icon'];
        $this->settings['iframe_height'] = $subOptionsettings['iframe_height'];
        $this->settings['default_charge_method'] = $this->payplus_default_charge_method;
    }

	public function payment_fields() {
		$description = $this->get_description();
		if ( $description ) {
			echo wpautop( wptexturize( $description ) ); // @codingStandardsIgnoreLine.
		}

		if ( $this->supports( 'default_credit_card_form' ) ) {
			$this->credit_card_form(); // Deprecated, will be removed in a future version.
		}
	}
    public function save_payment_method_checkbox(){}
    public function msg_checkout_code(){}
}

class WC_PayPlus_Gateway_Bit extends WC_PayPlus_Subgateway {
    public $id = 'payplus-payment-gateway-bit';
    public $method_title_text = 'Payplus - Bit';
    public $default_description_settings_text = 'Bit payment via PayPlus';
    public $method_descrition_text = 'Pay ith Bit via PayPlus';
    public $payplus_default_charge_method = 'bit';
    public $iconURL = 'assets/images/bit.png';
    public $pay_with_text = 'Pay with Bit';
}

class WC_PayPlus_Gateway_GooglePay extends WC_PayPlus_Subgateway {
    public $id = 'payplus-payment-gateway-googlepay';
    public $method_title_text = 'Payplus - GooglePay';
    public $default_description_settings_text = 'GooglePay payment via PayPlus';
    public $method_descrition_text = 'Pay ith Bit via PayPlus';
    public $payplus_default_charge_method = 'google-pay';
    public $iconURL = 'assets/images/google-pay.png';
    public $pay_with_text = 'Pay with GooglePay';
}
class WC_PayPlus_Gateway_ApplePay extends WC_PayPlus_Subgateway {
    public $id = 'payplus-payment-gateway-applepay';
    public $method_title_text = 'Payplus - ApplePay';
    public $default_description_settings_text = 'ApplePay payment via PayPlus';
    public $method_descrition_text = 'Pay ith Bit via PayPlus';
    public $payplus_default_charge_method = 'apple-pay';
    public $iconURL = 'assets/images/apple-pay.png';
    public $pay_with_text = 'Pay with ApplePay';
}