<?php
/*
Plugin Name: PayPlus Payment Gateway
Description: WooCommerce integration for PayPlus Payment Gateway. Accept credit cards and more alternative methods directly to your WordPress e-commerce websites, More options to Refund, Charge, Capture, Subscriptions, Tokens and much more!
Plugin URI: https://www.payplus.co.il/wordpress
Version: 1.4.7
Author: PayPlus LTD
Author URI: https://www.payplus.co.il/
License: GPLv2 or later
Text Domain: PayPlus Payment Gateway Plugin
*/

defined( 'ABSPATH' ) or die('Hey, You can\'t access this file!'); // Exit if accessed directly

define( 'PAYPLUS_PLUGIN_URL', plugins_url( '/', __FILE__ ) );
define( 'PAYPLUS_PLUGIN_DIR', dirname( __FILE__ ) );
define( 'PAYPLUS_VERSION', '1.4.7' );

class WC_PayPlus {
	
	protected static $instance = null;
	
	public static function get_instance() {
		if ( ! isset( static::$instance ) ) {
			static::$instance = new static;
		}
		
		return static::$instance;
	}
	
	public $notices = [];
	
	private function __construct() {
		add_action( 'admin_init', [ $this, 'check_environment' ] );
		add_action( 'admin_notices', [ $this, 'admin_notices' ], 15 );
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), [ $this, 'plugin_action_links' ] );
		add_action( 'plugins_loaded', [ $this, 'init' ] );
	}
	
	public function check_environment() {
		if ( is_admin() && current_user_can( 'activate_plugins' ) && ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			$message = __( 'This plugin requires <a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a> to be activated.', 'payplus-payment-gateway' );
			$this->add_admin_notice( 'error', $message );
			// Deactivate the plugin
			deactivate_plugins( __FILE__ );
			
			return;
		}
		
		$php_version          = phpversion();
		$required_php_version = '5.4';
		if ( version_compare( $required_php_version, $php_version, '>' ) ) {
			$message = sprintf( __( 'Your server is running PHP version %1$s but some features requires at least %2$s.', 'payplus-payment-gateway' ), $php_version, $required_php_version );
			$this->add_admin_notice( 'warning', $message );
		}
	}
	
	public function add_admin_notice( $type, $message ) {
		$this->notices[] = [
			'class'   => "notice notice-$type is-dismissible",
			'message' => $message
		];
	}
	
	public function admin_notices() {
		$output = '';
		$title  = __( 'PayPlus Payment Gateway', 'payplus-payment-gateway' );
		foreach ( $this->notices as $notice ) {
			$output .= "<div class='$notice[class]'><p><b>$title:</b> $notice[message]</p></div>";
		}
		echo $output;
	}
	
	public static function plugin_action_links( $links ) {
		$action_links = [
			'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=payplus-payment-gateway' ) . '" aria-label="' . __( 'View PayPlus Settings', 'payplus-payment-gateway' ) . '">' . __( 'Settings' ) . '</a>'
		];
		$links        = array_merge( $action_links, $links );
		
		return $links;
	}
	
	public function init() {
		load_plugin_textdomain( 'payplus-payment-gateway', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
		
		if (class_exists("WooCommerce")) {
			include_once( PAYPLUS_PLUGIN_DIR . '/includes/wc_payplus_gateway.php' );
			include_once( PAYPLUS_PLUGIN_DIR . '/includes/wc_payplus_subgateways.php' );
			include_once( PAYPLUS_PLUGIN_DIR . '/includes/wc_payplus_admin_payments.php' );
		}
		
		add_filter( 'woocommerce_payment_gateways', [ $this, 'add_payplus_gateway' ] );
	}
	
	public function add_payplus_gateway( $methods ) {
		$methods[] = 'WC_PayPlus_Gateway';
		$methods[] = 'WC_PayPlus_Gateway_Bit';
		$methods[] = 'WC_PayPlus_Gateway_GooglePay';
		$methods[] = 'WC_PayPlus_Gateway_ApplePay';
		
		return $methods;
	}
	
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'payplus-payment-gateway' ), '2.0' );
	}
	
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'payplus-payment-gateway' ), '2.0' );
	}
	
}

WC_PayPlus::get_instance();